/******************************************************************************/
//
// Name: 	Interfacing SDRAM with ADSP-21371 Processor
//
/******************************************************************************

(C) Copyright 2006 - Analog Devices, Inc.  All rights reserved.

File Name:		main.asm

Date Modified:	20 Nov 2006 - JJ Rev 1.0
				

Software:       VisualDSP++4.5

Purpose:	To demonstrate 32-bit SDRAM interface with ADSP-21371 processor	
				
*******************************************************************************/

#include <def21371.h>

#define DataCnt 500 //This is the number of words to be written to the SDRAM and read
					//back from the SDRAM
#define PLLSetlDelay 4096 //Delay needed to settle PLL

// Macros to use different combinations of core clock to sdram clock ratio
// Uncomment ONLY ONE of the following combinations
#define CCLK_SDCLK_RATIO_2_0 // SDCLK = 133 MHz for a core clock of 266 MHz

/* For other ratios, the value of RDIV should be re-calculated. Also, the SDCTL
   configuration value would change */
   
//#define CCLK_SDCLK_RATIO_2_5 // SDCLK = 133 MHz for a core clock of 325 MHz
//#define CCLK_SDCLK_RATIO_3_0
//#define CCLK_SDCLK_RATIO_3_5
//#define CCLK_SDCLK_RATIO_4_0


.section/dm seg_dmda;
	.var SourceData[DataCnt];
	.var DestData[DataCnt];
	.var flag0;
		
.section/pm seg_rth;
 	nop;nop;nop;nop;nop;jump start;

// External Port DMA Channel 0 interrupt service routine
.section/pm seg_epdma0;
	r0 = 1;
	dm(flag0) = r0;
	rti;
	rti;
		
.section/pm seg_pmco;
start:
	CALL InterruptSetUp;
	CALL Set_CoreCLK_SDRAMClk_Ratio; //Set the Core clock to SDRAM clock ratio
	CALL PLL_Configuration; //Configure PLL to derive specific core clock from clkin
	CALL SDRAM_Bank_Mapping; // Mapping Bank 0 to 32-bit SDRAM
	CALL SDC_Configuration;	 // Configure the processors SDRAM controller - SDC
	CALL SourceDataInit;	 //Initialize Source Data in processors internal SRAM
	CALL Config_Flg_ScopeTrigger;
	
	// dummy READ from SDRAM to start the power up sequence
	nop; nop; nop;
	r13 = dm(0x00200000); //This initiates the power on sequence of SDRAM
	
	/*Set Flasg0 : Tigger LA to capture Power Sequence*/
	//bit set FLAGS FLG0; //Drive HI
	/*---------*/
	
	nop;nop;nop;nop;nop;nop;nop;nop;nop;nop;nop;
	nop;nop;nop;nop;nop;nop;nop;nop;nop;nop;nop;

	//This is DMA mode Write to SDRAM and Read from SDRAM
	CALL IntToExtDMA;
	idle; //Wait for DMA completion Interrupt
	CALL ExtToIntDMA;
	idle; //Wait for DMA completion Interrupt
	jump(pc,0);
//##################################################################################
//^^^^^^^^^^^^^^^^^^^Int To Ext EP DMA Configuration subroutine^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
IntToExtDMA:
	r0 = 0;
	dm(DMAC0) = r0; //Clear DMAC0 - EP DMA Channel0 control register
	
	r0 = SourceData;
	dm(IIEP0) = r0; //Load Internal Memory Index 
	
	r0 = 1;
	dm(IMEP0) = r0; //Load Internal Memory Modifier
	
	r0 = DataCnt;
	dm(CEP0) = r0;  //Load Count : Number of words transfer
	
	r0 = 0x00200000;
	dm(EIEP0) = r0; //Load External Memory Index
	
	r0 = 1;
	dm(EMEP0) = r0; //Load External Modifier
	/* Flag0 driven HI to trigger LA - Only for test purpose */
	//bit set FLAGS FLG0; //Drive HI
	/*-------------------------------------------------------*/
	
	// Now setup the DMA control registers
	r0 = DEN | DFLSH | TRAN; //DMADR / TRAN is "1" : Write to External Memory
	dm(DMAC0) = r0;
	
	RTS;
//##################################################################################
	
//##################################################################################
//^^^^^^^^^^^^^^^^^^^Ext To Int EP DMA Configuration subroutine^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
ExtToIntDMA:

	r0 = 0;
	dm(DMAC0) = r0; //Clear DMAC0 - EP DMA Channel0 control register
	
	r0 = DestData;
	dm(IIEP0) = r0; //Load Internal Memory Index 
	
	r0 = 1;
	dm(IMEP0) = r0; //Load Internal Memory Modifier
	
	r0 = DataCnt;
	dm(CEP0) = r0;  //Load Count : Number of words transfer
	
	r0 = 0x00200000;
	dm(EIEP0) = r0; //Load External Memory Index
	
	r0 = 1;
	dm(EMEP0) = r0; //Load External Modifier
	
	/* Flag0 driven HI to trigger LA - Only for test purpose */
	bit set FLAGS FLG0; //Drive HI
	/*-------------------------------------------------------*/
	
	// Now setup the DMA control registers
	r0 = DEN | DFLSH; //DMADR is "0" : Write to Internal Memory
	dm(DMAC0) = r0;
	
	RTS;
//##################################################################################


	
//##################################################################################
//^^^^^^^^^^^^^^^^^^^PLL Configuration subroutine^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
PLL_Configuration:
	
	ustat2 = dm(PMCTL); 
	bit clr ustat2 PLLM63 | PLLD8; //Clear multiplier and divisor bits
	
	bit set ustat2 PLLM16;  /* set a multiplier of 64 and a divider of 3 */ 
	bit set ustat2 PLLBP;   /* Put PLL in bypass mode. */ 
	bit clr ustat2 DIVEN;   /* clear the DIVEN bit  */ 
	dm(PMCTL) = ustat2;   /* The DIVEN bit should be cleared while placing the PLL in bypass mode */ 
	
	//Delay - 
    //wait for PLL to lock at new rate (requirement for modifying multiplier only)
	lcntr = PLLSetlDelay, do pllwait until lce; 
	pllwait: nop; 
	//Delay - Done
	
	ustat2 = dm(PMCTL);     /* DIVEN bit returns as zero */ 
	bit clr ustat2 PLLBP;   /* take PLL out of Bypass, PLL is now at CLKIN*4 (CoreCLK = CLKIN * M/N = CLKIN* 16/4) */ 
	dm(PMCTL) = ustat2;     /* The DIVEN bit should be cleared while taking the PLL out of bypass mode */ 

	RTS;
//##################################################################################


//##################################################################################
//^^^^^^^^^^^^^^^^^^^^^Core clock to SDRAM clock ratio subroutine^^^^^^^^^^^^^^^^^^^	
Set_CoreCLK_SDRAMClk_Ratio:
	// Set the Core clock to SDRAM clock ratio
	ustat1 = dm(PMCTL);
	#ifdef CCLK_SDCLK_RATIO_2_0
		bit clr ustat1 SDCKR2 | SDCKR2_5 | SDCKR3 | SDCKR3_5 | SDCKR4;
		bit set ustat1 SDCKR2 | DIVEN;
	#endif
	#ifdef CCLK_SDCLK_RATIO_2_5
		bit clr ustat1 SDCKR2 | SDCKR2_5 | SDCKR3 | SDCKR3_5 | SDCKR4;
		bit set ustat1 SDCKR2_5 | DIVEN;
	#endif
	#ifdef CCLK_SDCLK_RATIO_3_0
		bit clr ustat1 SDCKR2 | SDCKR2_5 | SDCKR3 | SDCKR3_5 | SDCKR4;
		bit set ustat1 SDCKR3 | DIVEN;
	#endif
	#ifdef CCLK_SDCLK_RATIO_3_5
		bit clr ustat1 SDCKR2 | SDCKR2_5 | SDCKR3 | SDCKR3_5 | SDCKR4;
		bit set ustat1 SDCKR3_5 | DIVEN;
	#endif
	#ifdef CCLK_SDCLK_RATIO_4_0
		bit clr ustat1 SDCKR2 | SDCKR2_5 | SDCKR3 | SDCKR3_5 | SDCKR4;
		bit set ustat1 SDCKR4 | DIVEN;
	#endif
	dm(PMCTL) = ustat1;	
	
	RTS;
//##################################################################################	


//##################################################################################
//^^^^^^^^^^^^^^^^^^^^^^SDRAM Bank Mapping subroutine^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^	
SDRAM_Bank_Mapping:
	// Mapping Bank 0 to 32-bit SDRAM
	// Make sure that jumper is set appropriately so that MS0 is connected to 
	// chip select of 32-bit SDRAM device
	ustat1 = dm(EPCTL);
	bit set ustat1 B0SD;
	bit clr ustat1 B1SD|B2SD|B3SD;
	dm(EPCTL) = ustat1;
	RTS;
//##################################################################################	
	

//##################################################################################
//^^^^^^^^^^^^^^^^^^^^^SDC configuration subroutine^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
SDC_Configuration:
	// Programming SDRAM control registers.
	// RDIV = ((f SDCLK X t REF )/NRA) - (tRAS + tRP )
	ustat1 = 0x814; // (133*(10^6)*64*(10^-3)/4096) - (7+3) 
						// SDRAM clock at 133MHz

	/*For other CCLK_SDCLK ratio values, the RDIV value needs to be re-calculated based
	  on the SDRAM clock frequency. Above formula should be used for to calculate 
	  the RDIV value */
	

	bit set ustat1 SDROPT | BIT_17; // Enabling SDRAM read optimization
										// Setting the Modify to 1
	dm(SDRRC) = ustat1;
	
	ustat1 = 0;
	bit set ustat1 SDCL3|SDTRAS7|SDTRP3|SDCAW8|SDPSS|SDTWR2|SDTRCD3|SDRAW12;
	/*SDCTL value will depend on the SDRAM clock and hence, these values need to be
	  re-calculated for a different SDRAM clock frequency */
	
	dm(SDCTL) = ustat1;
	
	RTS;
//##################################################################################


//##################################################################################
//^^^^^^^^^^^^^^^^^^^^^^^Source data initialization subroutine^^^^^^^^^^^^^^^^^^^^^^
SourceDataInit:
	r0 = 0;
	i1 = SourceData;
	m1 = 1;
	lcntr = DataCnt, do SourceInit until lce;
	dm(i1,m1) = r0;
SourceInit:	r0 = r0 + 1;
	RTS;
//##################################################################################



//##################################################################################
//^^^^^^^^^^^^^^^^^^^^^^^Flg SetUp^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
Config_Flg_ScopeTrigger:
	bit set FLAGS FLG0O;//Configure FLAG0 output
	bit clr FLAGS FLG0; //Drive LOW
	//bit set FLAGS FLG0; //Drive HI
	RTS;

InterruptSetUp:
	// Enabling DMA interrupt
	bit set mode1 IRPTEN;
	bit set lirptl P9IMSK; //Enable programmable interrupt "9" - default: EPDMA 0 Interrupt
	nop;nop;nop;nop;
	RTS;
