#include <def21375.h>

.extern InitSRU;

#define PLLSetlDelay 4096 //Delay needed to settle PLL


.section/pm seg_rth;

//EMUI:
	nop;
	nop;
	nop;
	nop;
	
//RSTI:
	nop;
	jump start;
	nop;
	nop;
	
.section/pm seg_pmco;

start:
	Call Set_CoreCLK_SDRAMClk_Ratio; //Set the Core clock to SDRAM clock ratio
	call InitPLL;
	call InitSRU;
	call SDRAM_Bank_Mapping; // Mapping Bank 0 to 32-bit SDRAM
	call SDC_Configuration;	 // Configure the processors SDRAM controller - SDC
	
	// dummy READ from SDRAM to start the power up sequence
	nop; nop; nop;
	r13 = dm(0x00200000); //This initiates the power on sequence of SDRAM
	nop;nop;nop;nop;nop;nop;nop;nop;nop;nop;nop;
	nop;nop;nop;nop;nop;nop;nop;nop;nop;nop;nop;
	
	call blink_led;
	
	jump (pc,0);
	
	
InitPLL:

// CLKIN= 16.384 MHz, Multiplier= 16, CCLK_SDCLK_RATIO 2.
// Core clock = (16.384MHz * 16) = 262.144 MHz
    ustat2 = dm(PMCTL); 
	bit clr ustat2 PLLM63 | PLLD8; //Clear multiplier and divisor bits
	
	bit set ustat2 PLLM16;  /* set a multiplier of 16 */ 
	bit set ustat2 PLLBP;   /* Put PLL in bypass mode. */ 
	bit clr ustat2 DIVEN;   /* clear the DIVEN bit  */ 
	dm(PMCTL) = ustat2;   /* The DIVEN bit should be cleared while placing the PLL in bypass mode */ 
	
	//Delay - 
    //wait for PLL to lock at new rate (requirement for modifying multiplier only)
	lcntr = PLLSetlDelay, do pllwait until lce; 
	pllwait: nop; 
	//Delay - Done
	
	ustat2 = dm(PMCTL);     /* DIVEN bit returns as zero */ 
	bit clr ustat2 PLLBP;   /* take PLL out of Bypass, PLL is now at CLKIN*4 (CoreCLK = CLKIN * M/N = CLKIN* 16/4) */ 
	dm(PMCTL) = ustat2;     /* The DIVEN bit should be cleared while taking the PLL out of bypass mode */ 


InitPLL.end: rts;	


Set_CoreCLK_SDRAMClk_Ratio:
	// Set the Core clock to SDRAM clock ratio
	ustat1 = dm(PMCTL);
	#ifdef CCLK_SDCLK_RATIO_2_0
		bit clr ustat1 SDCKR2 | SDCKR2_5 | SDCKR3 | SDCKR3_5 | SDCKR4;
		bit set ustat1 SDCKR2 | DIVEN;
	#endif
	#ifdef CCLK_SDCLK_RATIO_2_5
		bit clr ustat1 SDCKR2 | SDCKR2_5 | SDCKR3 | SDCKR3_5 | SDCKR4;
		bit set ustat1 SDCKR2_5 | DIVEN;
	#endif
	#ifdef CCLK_SDCLK_RATIO_3_0
		bit clr ustat1 SDCKR2 | SDCKR2_5 | SDCKR3 | SDCKR3_5 | SDCKR4;
		bit set ustat1 SDCKR3 | DIVEN;
	#endif
	#ifdef CCLK_SDCLK_RATIO_3_5
		bit clr ustat1 SDCKR2 | SDCKR2_5 | SDCKR3 | SDCKR3_5 | SDCKR4;
		bit set ustat1 SDCKR3_5 | DIVEN;
	#endif
	#ifdef CCLK_SDCLK_RATIO_4_0
		bit clr ustat1 SDCKR2 | SDCKR2_5 | SDCKR3 | SDCKR3_5 | SDCKR4;
		bit set ustat1 SDCKR4 | DIVEN;
	#endif
	dm(PMCTL) = ustat1;	
	
	RTS;
//##################################################################################	


//##################################################################################
//^^^^^^^^^^^^^^^^^^^^^^SDRAM Bank Mapping subroutine^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^	
SDRAM_Bank_Mapping:
	// Mapping Bank 0 to 32-bit SDRAM
	// Make sure that jumper is set appropriately so that MS0 is connected to 
	// chip select of 32-bit SDRAM device
	ustat1 = dm(EPCTL);
	bit set ustat1 B0SD;
	bit clr ustat1 B1SD|B2SD|B3SD;
	dm(EPCTL) = ustat1;
	RTS;
//##################################################################################	
	

//##################################################################################
//^^^^^^^^^^^^^^^^^^^^^SDC configuration subroutine^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
SDC_Configuration:
	// Programming SDRAM control registers.
	// RDIV = ((f SDCLK X t REF )/NRA) - (tRAS + tRP )
	ustat1 = 0x814; // (133*(10^6)*64*(10^-3)/4096) - (7+3) 
						// SDRAM clock at 133MHz

	/*For other CCLK_SDCLK ratio values, the RDIV value needs to be re-calculated based
	  on the SDRAM clock frequency. Above formula should be used for to calculate 
	  the RDIV value */
	

	bit set ustat1 SDROPT | BIT_17; // Enabling SDRAM read optimization
										// Setting the Modify to 1
	dm(SDRRC) = ustat1;
	
	ustat1 = 0;
	/*SDCTL value will depend on the SDRAM clock and hence, these values need to be
	  re-calculated for a different SDRAM clock frequency */
	bit set ustat1 SDCL3|SDTRAS7|SDTRP3|SDCAW9|SDPSS|SDTWR2|SDTRCD3|SDRAW12|X16DE;
	
	dm(SDCTL) = ustat1;
	
	RTS;
//##################################################################################

.section/pm seg_sdram;
blink_led:	
	bit set FLAGS FLG3O | FLG4O | FLG5O | FLG6O | FLG7O | FLG8O; 

	ustat1 = FLAGS;
	bit set ustat1 FLG3 | FLG4 | FLG5;
	bit clr ustat1 FLG6 | FLG7 | FLG8;
	FLAGS = ustat1;
	
loophere:
	nop;
	nop;
	
	LCNTR = 0x00FF, do delay1 until lce;
	nop;
	LCNTR = 0xFFFF, do delay until lce;
delay: nop;	
delay1: nop;

	bit tgl FLAGS  FLG3 | FLG4 | FLG5 | FLG6 | FLG7 | FLG8; 
	
blink_led.end:	jump loophere;	
	
	